;---patch.pro, v. 0.9, last modified: 2014-10-16, Andrea La Camera
;+
; NAME:
;       PATCH
; PURPOSE:
;       PATCH program is an IDL software developed for the deconvolution of 
;       mono-pupil images with space-variant point-spread functions (PSFs). The 
;       input image is decomposed in partially overlapping sub-domains, the size 
;       of which will depend on the properties of the PSF. 
;       The reconstruction is obtained by applying a deconvolution method with 
;       boundary effects compensation to each sub-domain. The final object is 
;       formed by a mosaic of the partial reconstructed images, after the 
;       removal of the common parts of adjacent sub-domains.
;       Two algorithms are available: the well-known Richardson-Lucy and
;       the Scaled Gradient Projection (SGP) developed by S. Bonettini et al. 
;       More information and a tutorial can be found on 
;       http://www.airyproject.eu/Tutorials/Patch/
;
; CALLING SEQUENCE:
;       1) In order to start the Graphical User Interface (GUI) of the program 
;          and to set all parameters, please enter at IDL prompt:
;          PATCH > patch 
;       2) In order to directly launch the deconvolution step, enter at IDL prompt:
;          PATCH > patch, path_param_file
;          where 'path_param_file' is the path of the param_file.sav that 
;          contains all parameters necessary for the reconstruction (previously
;          saved from the GUI). 
;
; INPUTS:
;       mode 1) --> no input
;       mode 2) --> path_param_file in the form of "/home/andrea/Desktop/mypar.sav"
;
; OPTIONAL INPUT KEYWORD:
;       None
;
; OUTPUTS:
;       None
;
; RESTRICTIONS:
;       None
;
; MODIFICATION HISTORY:
;       WRITTEN, Andrea La Camera (DIBRIS, Univ. of Genova), November 2011
;       v0.81 - Minor modifications in the GUI  (2014-09-29, ALC)
;       v0.9  - Overlap management changed. Added a new routine for
;               this goal. Changes in patch_gui.pro and patch_deconv.pro (2014-10-16 ALC)
;-

pro patch, param_file

on_error, 2
COMMON image, img, header, N, M, a_dim, Nori, Mori
COMMON background, bg   ;bg must be N x M, as the input image "img" is.
COMMON output, out, res_array, residual   ; reconstructed object with original dimension Nori x Mori

if keyword_set(param_file) then begin
   ;message, "NOT YET IMPLEMENTED"
   ;load the parameters (param)
   restore, param_file, /VERBOSE
   ;define the structure 
   state={par:param}
   ; load image and define img, N, M
   a=patch_load_image(state, 0, VERBOSE=0)
   if a EQ 0 then message, 'ERROR on loading the input image file.'
   ;check if zero_padding is neccessary...
   if (state.par.cols*state.par.domx GT N) OR $
      (state.par.rows*state.par.domy GT M) then begin
       N=state.par.cols*state.par.domx
       M=state.par.rows*state.par.domy
       img = zero_padding(img, N, M)
   endif
   ;load (or define) the sky background array
   if state.par.bg_choice EQ 0 then begin
      bg = make_array(N,M,value=state.par.bg)
   endif else begin
      a=patch_load_background(state,0,VERBOSE=0)
      if a EQ 0 then message, 'ERROR on loading the background file.' 
   endelse
   
   ;RUN DECONVOLUTION!
   if n_elements(img) EQ 0 then message, 'Input image undefined!'
   if n_elements(bg) EQ 0 then message, 'Background undefined!'
   t0=systime(1)
   print, "Please, wait..."
   patch_deconv, state.par, /SAVE_OUT
   print, "...done!"
   print, "Total deconvolution time (s) = ", systime(1)-t0
   print, "Average time for each sub-domain = ", $
            (systime(1)-t0)/(state.par.rows*state.par.cols) 

endif else patch_gui

end
